const Listr = require('listr')
const co = require('co')
const createInstall = require('./create-install')
const getDeps = require('./get-deps')
const { filterNodeCorePkgs, filterDepsExisted } = require('./filter')
const exec = require('./exec')
const { hasPkgJsonHere } = require('./utils')
const PkgError = require('./pkg-error')

exports.tasks = co.wrap(function * (input, flags) {
  const tasks = new Listr([
    {
      title: 'Ensure if package.json exists',
      task: () => {
        if (!hasPkgJsonHere()) {
          throw new PkgError('package.json not exists, please create first !')
        }
      }
    },
    {
      title: 'Anylise the dependent pkgs',
      task: () => Promise.resolve('done')
    },
    {
      title: 'Filter the pkgs',
      skip: ctx => ctx.deps.length === 0,
      task: ctx => {
        ctx.deps = ctx.deps.filter(filterNodeCorePkgs).filter(filterDepsExisted)
      }
    },
    {
      title: 'Install pkgs',
      skip: ctx => {
        if (ctx.deps.length === 0) {
          return 'deps already exists'
        }
      },
      task: ctx => {
        const [cmd, ...rest] = createInstall({
          saveDev: ctx.opts.dev,
          deps: ctx.deps
        })
        return exec(cmd, rest)
      }
    }
  ])

  const opts = {
    dev: false
  }
  const deps = yield getDeps(input, flags)

  if (flags.d) {
    opts.dev = true
  }

  return tasks.run({ deps, opts })
})
